﻿// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.
// See the LICENSE file in the project root for more information.

/*============================================================
**
** 
** 
**
**
** Purpose: Exception for a missing satellite assembly needed
**          for ultimate resource fallback.  This usually
**          indicates a setup and/or deployment problem.
**
**
===========================================================*/

using System;
using System.Runtime.Serialization;

namespace System.Resources
{
    [Serializable]
    public class MissingSatelliteAssemblyException : SystemException
    {
        private String _cultureName;
        private const int COR_E_MISSINGSATELLITEASSEMBLY = 0;

        public MissingSatelliteAssemblyException()
            : base(SR.MissingSatelliteAssembly_Default)
        {
            HResult = COR_E_MISSINGSATELLITEASSEMBLY;
        }

        public MissingSatelliteAssemblyException(String message)
            : base(message)
        {
            HResult = COR_E_MISSINGSATELLITEASSEMBLY;
        }

        public MissingSatelliteAssemblyException(String message, String cultureName)
            : base(message)
        {
            HResult = COR_E_MISSINGSATELLITEASSEMBLY;
            _cultureName = cultureName;
        }

        public MissingSatelliteAssemblyException(String message, Exception inner)
            : base(message, inner)
        {
            HResult = COR_E_MISSINGSATELLITEASSEMBLY;
        }

        protected MissingSatelliteAssemblyException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public String CultureName
        {
            get { return _cultureName; }
        }
    }
}